<?php
namespace Teknisa\CTB\Controllers;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response;
use Teknisa\CTB\Factories\Ctb24000BalanceteFactory;

class Ctb00700RelatoriosContabeis extends \Zeedhi\Framework\Controller\Report {

    protected $novoCodigo;
    protected $procedureNutriente;
    protected $parameterBagColumns;
    protected $birtStrategy;
    protected $entityManager;
    protected $qr2Strategy;
    protected $nrsequnutr;
    protected $cdfilial;
    protected $nrorg;
    protected $connection;

    public function __construct($reportService, $novoCodigo, $entityManager, $qr2Strategy, $connection,Ctb24000BalanceteFactory $balanceteFactory) {

        $this->reportService      = $reportService;
        $this->novoCodigo         = $novoCodigo;
        $this->entityManager      = $entityManager;
        $this->qr2Strategy        = $qr2Strategy;
        $this->connection         = $connection;
        $this->balanceteFactory = $balanceteFactory;

    }

    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat($row["FORMAT"]);
    }
    
    public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $urlReport) 
    {
        //implementações necessárias pós relatório
        $this->urlReport = $urlReport;
        return $this->urlReport;
    }

    
    public function preparaParametroRelQr2(DTO\Request\Row $request, DTO\Response $response) {
    try{
        
        $row = $request->getRow();
      
        $reportName = $row["REPORT_CODE"];
        
        if($row["REPORT_CODE"] == 'CTB21010'){ // Diário da Contabilidade
            if(!$row["DIARIO"]){
                $nrdiario = '1';
            } else {
                $nrdiario = $row["DIARIO"];
            }
            
            $reportMapping = array(
                'NMEMPRESA' =>  $row["NMEMPRESA"],
                'CDEMPRESA' => $row["CDEMPRESA"],
                'STDATAINI' => $row["DTINIPERIODO"],
                'STDATAFIM' => $row["DTFIMPERIODO"],
                'DIARIO' => $nrdiario,
                'PAGINA' => $row["NRPAGINA"],
                'ULTPAGINA' => $row["ULTPAGINA"],
                'DTINICIAL' => $row["DTINIPERIODO"],
                'DTFINAL' => $row["DTFIMPERIODO"],
                'NRORG' => $row["NRORG"],
                "__format" => $row["TIPOREPORT"]
            );
             $type = 'BIRT';
            
        } if($row["REPORT_CODE"] == 'CTB23000'){ // Balancete Fiscal
            
            $cdempresa =$row["CDEMPRESA"];
            $cdcontini = $row["CDCONTCTBLINI"];
            $cdcontfin = $row["CDCONTCTBLFIN"];
            $datainic = $row["DTINIPERIODO"];
            $datafin = $row["DTFIMPERIODO"];
            $nivel = $row["CDNVCONT"];
            $encerramento = $row["DESCONSIDERA"];
            
            $balancete = $this->balanceteFactory->execProcBALANCETE_OFICIAL(
              $cdempresa,
              $cdcontini,
              $cdcontfin,
              $datainic,
              $datafin,
              $nivel,
              $encerramento
            );
            
            $reportMapping = array(
                'NRORG' =>  $row["NRORG"],
                'CDEMPRESA' => $row["CDEMPRESA"],
                'DATAINIC' => $row["DTINIPERIODO"],
                'DATAFIN' => $row["DTFIMPERIODO"],
                'NRBALANCETE' => $balancete,
                "__format" => $row["TIPOREPORT"]
            );
             $type = 'BIRT';
            
        }if($row["REPORT_CODE"] == 'CTB23001'){ // Balancete Geral
            
            $reportMapping = array(
                'P_NRORG' =>  $row["NRORG"],
                'P_CDEMPRESA' => $row["CDEMPRESA"],
                'P_DTINICIAL' => $row["DTINIPERIODO"],
                'P_CDCONTINI' =>  $row["CDCONTCTBLINI"],
                'P_CDCONTFIM' => $row["CDCONTCTBLFIN"],
                'P_IDTIPOSALDO' => $row["TIPODESALDO"],
                "__format" => $row["TIPOREPORT"]
            );
            $type = 'QR2';
        }if($row["REPORT_CODE"] == 'CTB21002'){ // Diário Paisagem
            
            $reportMapping = array(
                'P_DTINICIAL' => $row["DTINIPERIODO"],
                'P_DTFINAL' => $row["DTFIMPERIODO"],
                'P_STDATAINI' => $row["DTINIPERIODO"],
                'P_STDATAFIM' => $row["DTFIMPERIODO"],
                'P_DIARIO' => $row["DIARIO"],
                'P_PAGINA' => $row["NRPAGINA"],
                'P_ULTPAGINA' =>  $row["ULTPAGINA"],
                'CDEMPR' => $row["CDEMPRESA"],
                'P_NRORG' =>  $row["NRORG"],
                "__format" => $row["TIPOREPORT"]
            );
            $type = 'QR2';
            
        }if($row["REPORT_CODE"] == 'CTB23004'){ // Balancete com Totalizadores
            
            $reportMapping = array(
                'P_DTINICIAL' => $row["DTINIPERIODO"],
                'P_DTFINAL' => $row["DTFIMPERIODO"],
                'P_STEMPRESA' =>  $row["CDEMPRESA"],
                'P_CDCONTINI' => $row["CDCONTCTBLINI"],
                'P_CDCONTFIM' => $row["CDCONTCTBLFIN"],
                'P_IDENCERRAMENTO' => $row["DESCONSIDERA"],
                'P_IDTIPOSALDO' => $row["TIPODESALDO"],
                'P_NRORG' =>  $row["NRORG"],
                "__format" => $row["TIPOREPORT"]
            );
            $type = 'QR2';
        }
        
       // parent::createRemoteReport($type, $reportName, $reportMapping, $isUrl = false);
        $birtQr2Url = $this->reportService->createRemoteReport($type, $reportName, $reportMapping, $isUrl = false);
        $response->addMessage(new DTO\Response\Message($birtQr2Url));
        
    } catch (\Exception $e){
      $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
    }
  }
}