<?php
namespace Teknisa\CTB\Factories;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb13000RecalculoSaldoFiscalFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }

    /*
     * Executa procedure para o recálculo do saldo fiscal. [ATUALIZA_SALDO_GERAL].
     */
    public function recalculaSaldoFiscal($cdempresa, $idrecsaldosoc, $cdcontini, $cdcontfin, $dtinicial, $dtfinal = null) {
        $procedure = new StoredProcedure($this->connection, 'ATUALIZA_SALDO_GERAL');

        $procedure->addParam(new Param('P_CDEMPRESA', Param::PARAM_INPUT, $cdempresa, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCONTINI', Param::PARAM_INPUT, $cdcontini, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDCONTFIN', Param::PARAM_INPUT, $cdcontfin, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAINIC', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CALCSOC', Param::PARAM_INPUT, $idrecsaldosoc, Param::PARAM_TYPE_STR));

        $procedure->execute();
    }

    /*
     * Função para validar se uma determinada data inicial é igual ou menor que a data em que foi realizada o último recálculo.
     */
    public function validaDataInicial($params) {
        // Query a ser executada.
        $sql = "SELECT TO_CHAR(MAX(S.DTSALDEMPR), 'MM/YYYY') DATASALDO
                FROM SALDEMPR S, (SELECT LAST_DAY(ADD_MONTHS(TO_DATE(:DTREF, 'DD/MM/YYYY'), -1)) DATA
                                    FROM DUAL) X
               WHERE S.CDEMPRESA = :CDEMPRESA
                 AND S.NRORG     = :NRORG
              HAVING MAX(S.DTSALDEMPR) < X.DATA";

        // Prepara o statement da query para execução.
        $statement = $this->connection->prepare($sql);

        // Executa o statement com os paràmetros passados.
        $statement->execute($params);

        // Executa um fetch no primeiro dado, retornando-o.
        return $statement->fetch(\PDO::FETCH_ASSOC);
    }

}