<?php

namespace Teknisa\CTB\Services;

use Teknisa\CTB\Factories\ContabilizacaoRetiradaFactory;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

const tablePrxLancLote    = 'LANCLOTE';     //Tabela no qual será inserida a contabilização de estoque
const tablePrxMoviEstoque = 'MOVIESTOQUE';

class ContabilizacaoRetiradaService {

    protected $factory;
    protected $entityManager;
    protected $connection;
    protected $nrorg;
    protected $environment;
    protected $newCode;
    
    private $Messages = array(
        "Arq_Rel_Err" => "O arquivo contendo a definição do relatório de código '%s' não foi encontrado no diretório '%s'.",
        "NovoCaption" => "Título do relatório",
        "NovoPrompt" => "Informe o título do relatório:",
        "Filial_NCAD" => "Não existe filial para a empresa informada.",
        "Filial" => "Filial",
        "TipoReti_NCad" => "Tipo de retirada não cadastrado.",
        "TipoReti" => "Tipo de Retirada",
        "Filial_Inf" => "Filial deve ser informada.",
        "TipoReti_Inf" => "Tipo de Retirada deve ser informado.",
        "DataIni_Inf" => "Data inicial do período deve ser informada.",
        "DataFim_Inf" => "Data final do período deve ser informada.",
        "DtIni>DtFinal" => "Data inicial deve ser menor ou igual a data final do período.",
        "DtContab_Inf" => "Data de contabilização deve ser informada.",
        "TipoCusto_Inf" => "Tipo de custo deve ser informado.",
        "NaoDadoContab" => "Não há dados para contabilização.",
        "Empresa_Inf" => "Empresa deve ser informada.",
        "Empresa_Cad" => "Empresa não cadastrada.",
        "Lote_Inf" => "Lote deve ser informado.",
        "Lote_JCad" => "Lote já cadastrado.",
        "Contabilizacao" => "Contabilização das retiradas",
        "Contab_Ok" => "Contabilização das retiradas foi realizada com sucesso.",
        "Contab_NGerado" => "Contabilização das retiradas já foi gerada para um outro lote.",
        "Prod" => "Produto %s não possui conta contábil parametrizada para a filial %s.",
        "Prdt_CMV" => "Produto %s não possui conta contábil de CMV cadastrada.",
        "TpRet" => "Tipo de Retirada %s não possui conta contábil parametrizada.",
        "VERIFI_RETIRADAS" => "Verificando Retiradas Efetuadas...",
        "Lote_12" => "Operação Bloqueada! O Código de Lote 12 é Utilizado por uma Rotina Interna do Sistema.",
        /*da tela ger49100, sem as tags repetidas*/
        "TipoOper_NCad" => "Tipo de operação não cadastrado.",
        "TipoOper" => "Tipo de Operação",
        "Filial_Inv" => "Filial não cadastrada.",
        "TipoOper_Inf" => "Tipo de operação deve ser informado.",
        "ContabImp" => "Relatório da contabilização das implantações/ajustes.",
        "PRODSEMCONTA" => "Existem produtos sem contas contábeis parametrizadas. Favor emitir o relatório 'Produtos sem Conta Contábil' para conferência.",
        "EMP_DEV" => "Empresa deve ser informada.",
        "LOTE_DEV" => "Código do lote para contabilização deve ser informado.",
        "LOTE_JA" => "Lote já cadastrado para empresa informada. Favor informar outro código de lote.",
        "HIST_N" => "Histórico Padrão não cadastrado.",
        "HIST_DEV" => "Histórico Padrão deve ser informado.",
        "CONTAB_SUCESSO" => "Contabilização realizada com sucesso."
        );

    public function __construct(\Doctrine\ORM\EntityManager $entityManager, ContabilizacaoRetiradaFactory $factory, Environment $environment, NewCode $newCode) {
        $this->factory       = $factory;
        $this->entityManager = $entityManager;
        $this->connection    = $entityManager->getConnection();
        $this->environment   = $environment;
        $this->nrorg         = $environment->getNrOrgTrab();
        $this->newCode       = $newCode;
    }
    
    /*Retorna a mensagem de acordo com o código*/
    public function getMessage($msgCodigo, $msgComplemento = null, $params = null) {
        if(is_null($msgCodigo))
            return "";
        if(!isset($this->Messages[$msgCodigo]))
            return "";
        $msg = $this->Messages[$msgCodigo].$msgComplemento;
        if(!is_null($params)) {
            $msg = vsprintf($msg, $params);
        }
        return $msg;
    }
    
    /*Ajsuta um valor de acordo com a mascara (pict) selecionada do paramgeral*/
    public function setPictFormat($fieldPict, $text/*, $entityManager, $nrorg = null*/) {
        //Dados para a aquisicao do pict
        $sql = "SELECT CDPICTPROD AS 'CDPICTPROD', CDPICTRECE AS 'CDPICTRECE', CDPICTSERV AS 'CDPICTSERV', CDPICTCFIS AS 'CDPICTCFIS', CDPICTCCTB AS 'CDPICTCCTB', CDPICTETAPA AS 'CDPICTETAPA', CDPICTPRAT AS 'CDPICTPRAT' , CDPICTREFE AS 'CDPICTREFE', CDPICTDIET AS 'CDPICTDIET', CDPICTCFIN AS 'CDPICTCFIN', CDPICTINSPEC AS 'CDPICTINSPEC', CDPICTSGRU AS 'CDPICTSGRU', CDPICTESTAGI AS 'CDPICTESTAGI', CDPICTCFOP AS 'CDPICTCFOP', CDPICTPROCES AS 'CDPICTPROCES', CDPICTCCUSTO AS 'CDPICTCCUSTO', CDPICTITTRAB AS 'CDPICTITTRAB', CDPICTATIVCL AS 'CDPICTATIVCL', CDPICTBEM AS 'CDPICTBEM', CDPICTCLASSBEM AS 'CDPICTCLASSBEM' FROM PARAMGERAL WHERE(NRORG = :P_NRORG or :P_NRORG IS NULL)";
        $connection = $this->entityManager->getConnection();
        $statement = $connection->prepare($sql);
        $statement->execute(array("P_NRORG" => $this->nrorg));
        //Retorna o primeiro registro do paramgeral com os pict's
        $pict = $statement->fetch();

        $fieldPict = strtoupper($fieldPict);
        
        if($pict != "") { //Verificacoes de existencia de dados
            if(isset($pict[$fieldPict])) {
                //Adquiri o pict desejado
                $pict = $pict[$fieldPict];
                //Ajustes na mascara
                $pict = str_replace("a", "9", $pict);
                $pict = str_replace("1", "9", $pict);
                //Adquiri o tamanho da máscara
                $pict_size = count(explode("9", $pict)) - 1;
                //Ajuste final para o vsprintf
                $pict = str_replace("9", "%s", $pict);
                //Ajuste do tamanho da string de entrada, esta deve ter no minimo o tamanho da mascara
                $text = str_pad($text, $pict_size, " ", STR_PAD_LEFT);
                //Conversão da string de entrada para array
                $text = str_split($text);
                //Retorna a string formatada
                $text = vsprintf($pict, $text);
                return $text;
            }
        }
        //Se chegou aqui significa que não há pict para os parâmetros selecionados
        return null;
    }
    
    /*Gera um novo sequencial. O cdcontador é a concatenação da tabela, cod empresa e cod do lote*/
    public function PrxLancLote($cdempresa, $cdlote) {
        $cod = substr(tablePrxLancLote.$cdempresa.$cdlote, 0, 100); //Limita o campo a 100 posições (tamanho do CDCONTADOR em NOVOCODIGO)
        
        return $this->newCode->getCodeByTableName($cod, $this->nrorg, 1, 0, true);
    }
    
    /*Gera um novo sequencial para a movimentação de estoque*/
    public function PrxMoviEstoque() {
        return $this->newCode->getCodeByTableName(tablePrxMoviEstoque, $this->nrorg, 1, 10);
    }
    
    /*Realiza o rollback*/
    public function rollback() {
        $this->factory->rollback();
    }

    /*Gera o Lote - $dsObseLotePadrao é a descrição padrão para o campo DSOBSELOTE da tabela CAPALOTE*/
    public function geraLote($row, $dsObseLotePadrao) {
        $result_GeraLote               = [];
        $result_GeraLoteDtMovi         = [];
        $result_GeraLoteCustLanc       = [];
        $result_GeraLoteCustLancDtMovi = [];
        $result_VerLote                = [];
        $result_ExistLancLote          = [];
        $cdsResult                     = [];
        $cdsGeraLote                   = [];
        $Result                        = '';
        
        /*** Funçaõ do GetGeraLote ***/
        $result_ContOpeCtbl = $this->factory->adqContOpeCtbl($row["CDFILIAL"], $row["CDEMPRESA"], $this->nrorg);

        //Adquiri o $cdsResult
        foreach($result_ContOpeCtbl as $ContOpeCtbl) {
            
            if($ContOpeCtbl["IDUTCUSTESTCONT"] == 'S' || $ContOpeCtbl["IDUTCUSTESTCONT"] == '') {
                
                //Obtem o GeraLote ou GeraLoteDtMovi, Custo do Estoque
                if($ContOpeCtbl["IDCONSDTMOVIRETEST"] == 'N') {
                    $result_GeraLote = $this->factory->adqGeraLote($ContOpeCtbl["CDFILIAL"], $row["CDTIPORETI"], $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], $row["DTPERIODO"]["end"], $row["CDEMPRESA"], $this->nrorg);
                } else {
                    $result_GeraLoteDtMovi = $this->factory->adqGeraLoteDtMovi($ContOpeCtbl["CDFILIAL"], $row["CDTIPORETI"], $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], $row["CDEMPRESA"], $this->nrorg, $this->nrorg);
                }

                //Armazena apenas os dados existentes de Custo do Estoque (GeraLote e GeraLoteDtMovi) em cdsResult
                if(count($result_GeraLote) > 0) {
                    foreach($result_GeraLote as $val) {
                        $cdsResult[] = $val;
                    }
                } elseif(count($result_GeraLoteDtMovi) > 0) {
                    foreach($result_GeraLoteDtMovi as $val) {
                        $cdsResult[] = $val;
                    }
                }
            } else {
                //Obtem o GeraLoteCustLanc ou GeraLoteCustLancDtMovi, Custo do Lançamento Estoque
                if($ContOpeCtbl["IDCONSDTMOVIRETEST"] == 'N') {
                    $result_GeraLoteCustLanc = $this->factory->adqGeraLoteCustLanc($ContOpeCtbl["CDFILIAL"], $row["CDTIPORETI"], $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], $row["DTPERIODO"]["end"], $row["CDEMPRESA"], $this->nrorg);
                } else {
                    $result_GeraLoteCustLancDtMovi = $this->factory->adqGeraLoteCustLancDtMovi($ContOpeCtbl["CDFILIAL"], $row["CDTIPORETI"], $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], $row["CDEMPRESA"], $this->nrorg);
                }
                
                //Armazena apenas os dados existentes de Custo do Lançamento Estoque (GeraLoteCustLanc e GeraLoteCustLancDtMovi) em cdsResult
                if(count($result_GeraLoteCustLanc) > 0) {
                    foreach($result_GeraLoteCustLanc as $val) {
                        $cdsResult[] = $val;
                    }
                } elseif(count($result_GeraLoteCustLancDtMovi) > 0) {
                    foreach($result_GeraLoteCustLancDtMovi as $val) {
                        $cdsResult[] = $val;
                    }
                }
            }
        } //Fim do loop adqContOpeCtbl
        
        /*** Função do Set Confirma ***/
        //Trata o $cdsResult
        $result_VerLote = $this->factory->adqVerLote($row["CDEMPRESA"], $row["CDLOTE"], $this->nrorg); //479
        if(count($result_VerLote) != 0) { //486
            throw new \Exception($this->getMessage("Lote_JCad"));
        } else { //489
            $cdsGeraLote = $cdsResult; //Para manter a nomeclatura do código original
            unset($cdsResult);
            $result_ExistLancLote = $this->factory->adqExistLancLote($row["CDEMPRESA"], $row["CDLOTE"], $this->nrorg);
            $stNrSequLote = '0'; //499
            $stLancamento = $row["CDEMPRESA"].$row["CDLOTE"];
            $stNrLancLote = $this->PrxLancLote($row["CDEMPRESA"], $row["CDLOTE"]);

            foreach($cdsGeraLote as $cdsGeraLote_reg) { //510
                if(count($result_ExistLancLote) == 0) { //521
                    if($stNrSequLote == 0) { //522
                        $this->factory->acmCapaLote($row["CDLOTE"], $row["CDEMPRESA"], $dsObseLotePadrao, 'D', $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], $this->nrorg); //523
                    }
                    
                    $stNrSequLote++; //533
                    if($cdsGeraLote_reg["CDCONTCTBL"] == '') { //536
                        if($cdsGeraLote_reg["IDCONTABCMV"] == 'S') { //537
                            throw new \Exception($this->getMessage("Prdt_CMV", " ", array($this->setPictFormat("CDPICTPROD", $cdsGeraLote_reg["CODIGOPROD"]).'  -  '.$cdsGeraLote_reg["NOMEPROD"]))); //541
                        } else {
                            throw new \Exception($this->getMessage("TpRet", " ", array($cdsGeraLote_reg["CDTIPORETI"])));
                        }
                        //break; //548 //tornou-se desnecessário devido ao throw acima
                    }
                    $stValor = $cdsGeraLote_reg["VRCUSTPROD"]; //551
                    $exValor = round((float)$cdsGeraLote_reg["VRCUSTPROD"], 2);
                    if($stValor != "" && $exValor <> 0) { //554

                        $this->factory->acmLancLote($row["CDEMPRESA"], $row["CDLOTE"], $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDCONTCTBL"], $cdsGeraLote_reg["CDHISTPADR"], $cdsGeraLote_reg["DTCONTAB"], '9999999999', $cdsGeraLote_reg["DSHISTPADR"], $exValor * -1, $this->nrorg);
                        
                        $result_adqContCtbl = $this->factory->adqContCtbl($cdsGeraLote_reg["CDCONTCTBL"], $this->nrorg); //569

                        if($cdsGeraLote_reg["CDCENTCUSTDB"] != "") { //575
                            if($result_adqContCtbl[0]["IDCONTCCUS"] != '0') {
                                $this->factory->acmRatCusto($row["CDEMPRESA"], $row["CDLOTE"], $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDFILIMOVI"], $cdsGeraLote_reg["CDCENTCUSTDB"], $cdsGeraLote_reg["DTCONTAB"], $exValor * -1, $this->nrorg);
                            }
                        }

                        $stNrSequLote++; //591
                        
                        if($cdsGeraLote_reg["CDCTBLENTRPR"] == "") { //598
                            //Query que busca produto com conta contábil nula
                            $result_adqProdSemCContabil = $this->factory->adqProdSemCContabil($row["CDFILIAL"], $row["CDTIPORETI"], $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], null, $this->nrorg); //599
                            //mensagem de erro: "Produto (codigo_produto) - (nome_produto) não possui conta contábil parametrizada":
                            //setPictFormat: formata a mascara de código de produto.
                            throw new \Exception($this->getMessage("Prod", null, array($cdsGeraLote_reg["NOMEPROD"], $cdsGeraLote_reg["CDFILIMOVI"]/*, $this->setPictFormat("CDPICTPROD", $cdsGeraLote_reg["CODIGOPROD"])*/))); //612
                            //break; //614 //tornou-se desnecessário devido ao throw acima
                        } else { //se a conta contábil não for nula, entra aqui e a opeação segue normalmente //616
                            $this->factory->acmLancLote($row["CDEMPRESA"], $row["CDLOTE"], $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDCTBLENTRPR"], $cdsGeraLote_reg["CDHISTPADR"], $cdsGeraLote_reg["DTCONTAB"], '9999999999', $cdsGeraLote_reg["DSHISTPADR"], $exValor, $this->nrorg); //592 //617
                        }
                        
                        $result_adqContCtbl = $this->factory->adqContCtbl($cdsGeraLote_reg["CDCTBLENTRPR"], $this->nrorg); //627

                        if($cdsGeraLote_reg["CDCENTCUSTDB"] != "") { //633
                            if($result_adqContCtbl[0]["IDCONTCCUS"] != "O") {
                                /*alterado para colocar o rateio para o mesmo centro de custo que a partida de débito,
                                pois o centro de custo a crédito vinha nulo, gerando assim erro de FK na rateccus*/
                                $this->factory->acmRatCusto($row["CDEMPRESA"], $row["CDLOTE"], $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDFILIMOVI"], $cdsGeraLote_reg["CDCENTCUSTDB"], $cdsGeraLote_reg["DTCONTAB"], $exValor, $this->nrorg); //637
                            }
                        }
                    }
                }

                if($stLancamento <> ($row["CDEMPRESA"].$row["CDLOTE"])) { //653
                    $stLancamento = $row["CDEMPRESA"].$row["CDLOTE"];
                    $result_ExistLancLote = $this->factory->adqExistLancLote($row["CDEMPRESA"], $row["CDLOTE"], $this->nrorg);
                }
            }

            if(count($cdsGeraLote) != 0 && $stNrSequLote != 0) { //664
                $this->factory->acmUpdateItLancto($row["CDFILIAL"], $row["CDTIPORETI"], $row["CDEMPRESA"], $row["CDLOTE"], $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], $this->nrorg);
            }
            
            $this->factory->commit(); //677

            if($stNrSequLote != "0") { //679
                $Result =  $this->getMessage("Contab_Ok");
            } else {
                $Result = $this->getMessage("Contab_NGerado");
            }
        }
        return $Result;
    }
    
    /*Gera o relatório de Apropriação Contábil Implantações, parte do cliente, arquivo ger49100.pas*/
    public function Prepara_Impressao($row, $p_geralote = 'N') {
        //.pas do GER49100, client

        $p_cdempresa = $row["CDEMPRESA"];
        $p_cdfilial  = $row["CDFILIAL"];
        $p_dtinicial = $row["DTPERIODO"]["start"];
        $p_dtfinal   = $row["DTPERIODO"]["end"];
        $p_idtplanc  = $row["IDTPLANC"];
        $p_cdlote    = $row["CDLOTE"];
        $AnaliSint   = $row["RELATORIO"];   /*A Apropriação Contábil Analítica Implantações/Ajustes
                                              S Apropriação Contábil Sintética Implantações/Ajustes
                                              C Produtos sem Conta Contábil*/ //478
        $cdsGeraLote      = []; //Array do resultado de saída (para o datasource)
        $cdsRelatorioAnal = [];
        $cdsRelatorioSin  = [];
        $cdsProdSContCtbl = [];
        
        if($p_geralote == 'S') { //269
            $this->VdCodLote($p_cdempresa, $p_cdlote); //Verifica se já existe um lote cadastrado para o valor informado pelo usuário, lança exceção se necessário
        }
        
        //$stPrxMoviEstoque = ''; //314 //Parece que não está sendo usado
        $arResult = ''; //273
        
        //$result_qryFiliaisEmpr = $this->factory->qryFiliaisEmpr($p_cdempresa, $p_cdfilial, $this->nrorg); //307
        foreach(/*$result_qryFiliaisEmpr*/$p_cdfilial as $cdfilial) {
            $arResult = $this->SetPreparaImp($p_cdempresa, $cdfilial, $p_dtinicial, $p_dtfinal, $p_idtplanc, $AnaliSint, $p_geralote); //310

            //$stPrxMoviEstoque = $arResult[0]; //314 //Parece que não está sendo usado
            $cdsGeraLoteAux      = isset($arResult[1]) ? $arResult[1] : array(); //316
            $cdsRelatorioAnalAux = isset($arResult[2]) ? $arResult[2] : array();
            $cdsRelatorioSinAux  = isset($arResult[3]) ? $arResult[3] : array();
            $cdsProdSContCtblAux = isset($arResult[4]) ? $arResult[4] : array();

            if(count($cdsGeraLoteAux) != 0) { //321
                foreach($cdsGeraLoteAux as $val) {
                    $cdsGeraLote[] = [  "CDFILIMOVIEST" => $val["CDFILIMOVIEST"],
                                        "NRLANCMOVIEST" => $val["NRLANCMOVIEST"],
                                        "CDCONTCTBL"    => $val["CDCONTCTBL"],
                                        "CDCTBLENTRPR"  => $val["CDCTBLENTRPR"],
                                        "CDCENTCUST"    => $val["CDCENTCUST"], //!!
                                        "VRCUSTPROD"    => $val["VRCUSTPROD"] ];
                }
            }
            
            if(count($cdsRelatorioAnalAux) != 0) { //336
                foreach($cdsRelatorioAnalAux as $val) {
                    $cdsRelatorioAnal[] = [ "DTINICIAL"     => $val["DTINICIAL"], //mestre
                                            "DTFINAL"       => $val["DTFINAL"],
                                            "DATACONTAB"    => $val["DATACONTAB"],
                                            "DTCONTAB"      => $val["DTCONTAB"],
                                            "NMTIPOCUSTO"   => $val["NMTIPOCUSTO"],
                                            "CDFILIMOVIEST" => $val["CDFILIMOVIEST"],
                                            "CDPRODMOVIEST" => $val["CDPRODMOVIEST"],
                                            "DTLANCMOVIEST" => $val["DTLANCMOVIEST"],
                                            "QTLANCTOEST"   => $val["QTLANCTOEST"],
                                            "NRLANCMOVIEST" => $val["NRLANCMOVIEST"],
                                            "NMPRODUTO"     => $val["NMPRODUTO"],
                                            "CDCONTCTBL"    => $val["CDCONTCTBL"],
                                            "CDCTBLENTRPR"  => $val["CDCTBLENTRPR"],
                                            "IDTIPOMOVIEST" => $val["IDTIPOMOVIEST"],
                                            "VRCUSTPROD"    => $val["VRCUSTPROD"],
                                            "VRUNIT"        => $val["VRUNIT"],
                                            "NMCONTCTBL"    => $val["NMCONTCTBL"],
                                            "NMCTBLENTRPR"  => $val["NMCTBLENTRPR"],
                                            "NMFILIAL"      => $val["NMFILIAL"] ];
                }
            }
            
            if(count($cdsRelatorioSinAux) != 0) { //363
                foreach($cdsRelatorioSinAux as $val) {
                    $cdsRelatorioSin[] = [  "DTINICIAL"     => $val["DTINICIAL"],
                                            "DTFINAL"       => $val["DTFINAL"],
                                            "DATACONTAB"    => $val["DATACONTAB"],
                                            "DTCONTAB"      => $val["DTCONTAB"],
                                            "NMTIPOCUSTO"   => $val["NMTIPOCUSTO"],
                                            "NMCONTCTBL"    => $val["NMCONTCTBL"],
                                            "CDFILIMOVIEST" => $val["CDFILIMOVIEST"],
                                            "CDCONTCTBL"    => $val["CDCONTCTBL"],
                                            "CDCTBLENTRPR"  => $val["CDCTBLENTRPR"],
                                            "VRCUSTPROD"    => $val["VRCUSTPROD"],
                                            "NMCTBLENTRPR"  => $val["NMCTBLENTRPR"],
                                            "NMFILIAL"      => $val["NMFILIAL"] ];
                }
            }
            
            if(count($cdsProdSContCtblAux) != 0) { //384
                foreach($cdsProdSContCtblAux as $val) {
                    $cdsProdSContCtbl[] = [ "CDFILIMOVIEST" => $val["CDFILIMOVIEST"],
                                            "CDARVPROD"     => $val["CDARVPROD"], //!!
                                            "CONTAAJUSTE"   => $val["CONTAAJUSTE"], //!!
                                            "CONTAIMPL"     => $val["CONTAIMPL"], //!!
                                            "CONTAENTRADA"  => $val["CONTAENTRADA"], //!!
                                            "NMPRODUTO"     => $val["NMPRODUTO"] ];
                }
            }
        } //400
        
        return array(   "cdsGeraLote"      => $cdsGeraLote,
                        "cdsRelatorioAnal" => $cdsRelatorioAnal,
                        "cdsRelatorioSin"  => $cdsRelatorioSin,
                        "cdsProdSContCtbl" => $cdsProdSContCtbl); //Retorna todas as CDS's, no controller (e em outras funções) será decidido qual será usada no datasource do relatório
    }
    
    /*Usada no geraRelatorioImplantacao(), parte do servidor, arquivo ger49100.pas*/
    private function SetPreparaImp($p_cdempresa, $p_cdfilial, $p_dtinicial, $p_dtfinal, $p_idtplanc, $p_analisint, $p_geralote) { //284
        /*Comentário do Delphi: A variável arResult é utilizada para que a função CheckResultServ do cliente não mande uma menssagem na tela com o valor da variável*/
        $arResult = array(null, null, null, null, null); //359
        
        //Bloco da Procedure MovimentacaoEstoque, parte do servidor, arquivo ger49100.pas //312
        $result_adqCodProd = $this->factory->adqCodProd($this->nrorg);
        $stPrxMoviEstoque = $this->PrxMoviEstoque(); //318
        
        //$result_adqFiliaisEmpr = $this->factory->adqFiliaisEmpr($p_cdempresa, $p_cdfilial, $this->nrorg); //322
        /*if(count($result_adqFiliaisEmpr) != 0) { //328
            foreach($result_adqFiliaisEmpr as $adqFiliaisEmpr_reg) {*/
                $this->factory->adsMovi_EstoqueORA(
                    /*$adqFiliaisEmpr_reg["CDFILIAL"]*/$p_cdfilial,
                    null,               //cdalmoxarife
                    null,               //cdlocalestoq
                    null,                 //nrloteestq
                    1,                  //tamnivel
                    $p_dtinicial,
                    $p_dtfinal,
                    $result_adqCodProd[0]["PRODINI"],
                    $result_adqCodProd[0]["PRODFIN"],
                    3,                  //idlancto
                    1,                  //idcusto
                    $stPrxMoviEstoque,  //sequmovi
                    'S',                //movconsolid
                    'N',                //delmovimen
                    $this->nrorg);
            /*}
        }*/
        //Fim do Bloco da Procedure MovimentacaoEstoque //360
        $arResult[0] = $stPrxMoviEstoque; //361
        $arResult[1] = $p_geralote == 'S' ? $this->factory->adqGeraLote2($p_cdfilial, $stPrxMoviEstoque, $p_idtplanc, $p_cdempresa, $this->nrorg)
                                          : null; //362 //364 //dspGeraLote.Data //378

        if($p_geralote != 'S') { //380
            //Relatórios Analíticos
            if($p_analisint == 'A') {
                if($p_idtplanc == 'A') {
                    $arResult[2] = $this->factory->adqRelatorioAnalAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', $stPrxMoviEstoque, $p_cdfilial, $p_cdempresa, $this->nrorg); //384
                } else {
                    $arResult[2] = $this->factory->adqRelatorioAnalImp($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', $stPrxMoviEstoque, $p_cdfilial, $p_cdempresa, $this->nrorg); //388
                }
                /*Comentário no Delphi: Abre as querys desnecessárias sem parâmetro para que o .Data não retorne nulo. Pode ser qualquer query (Ajuste ou Implantacao), pois os campos no CDS são o smesmos*/
                $arResult[3] = $this->factory->adqRelatorioSinAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', '', $p_cdfilial, $p_cdempresa, $this->nrorg); //393
                $arResult[4] = null; //adqProdSContCtbl //395
            } else {
                if($p_analisint == 'S') { //Relatórios Sintéticos //398
                    if($p_analisint == 'A') { //399
                        $arResult[3] = $this->factory->adqRelatorioSinAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', $stPrxMoviEstoque, $p_cdfilial, $p_cdempresa, $this->nrorg); //400
                    } else {
                        $arResult[3] = $this->factory->adqRelatorioSinImp($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', $stPrxMoviEstoque, $p_cdfilial, $p_cdempresa, $this->nrorg); //403
                    }
                    /*Comentário no Delphi: Abre as querys desnecessárias sem parâmetro para que o .Data não retorne nulo. Pode ser qualquer query (Ajuste ou Implantacao), pois os campos no CDS são o smesmos*/
                    $arResult[2] = $this->factory->adqRelatorioAnalAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', '', $p_cdfilial, $p_cdempresa, $this->nrorg); //409
                    $arResult[4] = null; //adqProdSContCtbl //411
                } else { //414
                    $arResult[4] = $this->factory->adqProdSContCtbl($p_cdfilial, $stPrxMoviEstoque, $p_idtplanc, $p_cdempresa, $this->nrorg); //415 //423
                    $arResult[2] = $this->factory->adqRelatorioAnalAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', '', $p_cdfilial, $p_cdempresa, $this->nrorg); //424
                    $arResult[3] = $this->factory->adqRelatorioSinAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', '', $p_cdfilial, $p_cdempresa, $this->nrorg); //426
                }
            }
        } else { //430
            $arResult[2] = $this->factory->adqRelatorioAnalAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', '', $p_cdfilial, $p_cdempresa, $this->nrorg); //431
            $arResult[3] = $this->factory->adqRelatorioSinAj($p_dtinicial, $p_dtfinal, $p_dtfinal/*datacontab*/, $p_dtfinal/*dtcontab*/, 'Líquido', '', $p_cdfilial, $p_cdempresa, $this->nrorg); //433
            $arResult[4] = $this->factory->adqProdSContCtbl($p_cdfilial, $stPrxMoviEstoque, $p_idtplanc, $p_cdempresa, $this->nrorg); //435
        }
        
        return $arResult; //445
    }
    
    /*Validação de lote. Verifica se já existe um lote cadastrado para o valor informado pelo usuário.*/
    private function VdCodLote($p_cdempresa, $p_cdlote) { //667 client //763 server
        $Result = $this->factory->adqVerLote($p_cdempresa, $p_cdlote, $this->nrorg);
        if(count($Result) != 0) {
            throw new \Exception($this->getMessage("LOTE_JA"));
        }
    }
    
    /*Gera a contabilização de Ajustes e Implantações, parte do cliente, arquivo ger49100.pas*/
    public function sbtContabClick($row) {
        $result = $this->Prepara_Impressao($row, 'S'); //442
        
        if(count($result["cdsProdSContCtbl"]) != 0) { //443
            throw new \Exception($this->getMessage("PRODSEMCONTA"));
        }
        
        if(count($result["cdsGeraLote"]) == 0) { //446
            throw new \Exception($this->getMessage("NaoDadoContab"));
        }
        
        $stTipoMovi = $row["IDTPLANC"] == 'I' ? '0' : '1'; //450
        
        $arResult = $this->SetConfirma($row["CDEMPRESA"], $row["CDLOTE"], $row["DTPERIODO"]["start"], $row["DTPERIODO"]["end"], $row["CDFILIAL"],
            $result["cdsGeraLote"], $row["CDHISTPADR"], $stTipoMovi); //457
            
        if($arResult != null) { //462
            if($arResult == 'Contab_Ok') { //463
                return $this->getMessage('CONTAB_SUCESSO');
            }
            if($arResult != 'Lote_JCad' && $arResult != 'Contab_Ok') { //466
                return $this->getMessage($arResult);
            }
        }
    }
    
    /*parte do servidor, arquivo ger49100.pas.*/
    private function SetConfirma($p_cdempresa, $p_cdlote, $p_dtinicial, $p_dtfinal, $p_cdfilimovi, $p_geralote, $p_cdhistpadr, $p_tipomovi) { //455
        $cdsGeraLote = $p_geralote; //470
        
        $this->factory->acmCapaLote($p_cdlote, $p_cdempresa, $this->getMessage($p_tipomovi)/*dsobselote*/, 'D'/*idlote*/, $p_dtinicial, $p_dtfinal, $this->nrorg); //472
        $stNrSequLote = '00000'; //481
        //Atualiza saldo do estoque referente as transferências de estoque
        $this->factory->adsAtuSaldTransf($p_cdfilimovi, $p_dtinicial, $p_dtfinal); //484
        
        //A numeração das linhas (no comentários) podem não estar precisas deste ponto em diante pois o código pas foi atualizado/corrigido
        //$cdsContaContabil = $this->factory->adqContaContabil($this->nrorg); //503 //Será usado dentro do foreach com a passagem de parâmetro extra CDCONTCTBL
        $dtContab = $p_dtfinal; //504
        
        foreach($cdsGeraLote as $key => $cdsGeraLote_reg) {  //505
            
            $result_adqExistLancLote2 = $this->factory->adqExistLancLote2($cdsGeraLote_reg["CDFILIMOVIEST"], $cdsGeraLote_reg["NRLANCMOVIEST"], $this->nrorg); //492
            $stNrLancLote = $this->PrxLancLote($p_cdempresa, $p_cdlote); //499
            //$stLancamento = $cdsGeraLote_reg["CDFILIMOVIEST"].$cdsGeraLote_reg["NRLANCMOVIEST"]; //500
            
            if(count($result_adqExistLancLote2) == 0) { //506
                $exValor = round($cdsGeraLote_reg["VRCUSTPROD"], 2); //507
                if($exValor != 0) { //508
                    //Conta contábil de Ajuste ou Implantação
                    $result_adqLancLote = $this->factory->adqLancLote($p_cdlote, $cdsGeraLote_reg["CDCONTCTBL"], $p_cdempresa, $stNrLancLote, $this->nrorg); //510
                    
                    if(count($result_adqLancLote) == 0) { //518
                        $stNrSequLote = str_pad($stNrSequLote + 1, 5, "0", STR_PAD_LEFT); //519
                        $stUltInsercao = $stNrSequLote;
                        $this->factory->acmLancLote($p_cdempresa, $p_cdlote, $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDCONTCTBL"], $p_cdhistpadr,
                            $dtContab, '9999999999', $this->getMessage($p_tipomovi), round($exValor, 2), $this->nrorg); //521
                    } else {
                        $stNrSequLote = $result_adqLancLote["NRSEQLANLOTE"]; //536
                        $this->factory->acmUpdateLancLote(round($exValor, 2), $p_cdempresa, $p_cdlote, $stNrLancLote, $cdsGeraLote_reg["CDCONTCTBL"], $this->nrorg); //537
                        $stNrSequLote = $stUltInsercao;
                    }
                    
                    $cdsContaContabil = $this->factory->adqContaContabil($cdsGeraLote_reg["CDCONTCTBL"], $this->nrorg); //549
                    $cdsContaContabil = $cdsContaContabil[0]; //O adqContaContabil retornará apenas um resultado quando este tiver CDCONTCTBL setado
                    
                    if($cdsGeraLote_reg["CDCENTCUST"] != "") { //553
                        if($cdsContaContabil["IDCONTCCUS"] != 'O') { //554
                            if(count($result_adqLancLote) == 0) { //555
                                $this->factory->acmRatCusto($p_cdempresa, $p_cdlote, $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDFILIMOVIEST"],
                                    $cdsGeraLote_reg["CDCENTCUST"], $dtContab, round($exValor, 2), $this->nrorg); //556
                            } else { //568
                                $result_adqRateCCus = $this->factory->adqRateCCus($p_cdempresa, $p_cdlote, $stNrLancLote, $cdsGeraLote_reg["CDCONTCTBL"],
                                    $cdsGeraLote_reg["CDFILIMOVIEST"], $cdsGeraLote_reg["CDCENTCUST"], $this->nrorg); //569
                                if(count($result_adqRateCCus) == 0) { //579
                                    $this->factory->acmRatCusto($p_cdempresa, $p_cdlote, $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDFILIMOVIEST"],
                                        $cdsGeraLote_reg["CDCENTCUST"], $dtContab, round($exValor, 2), $this->nrorg); //580
                                } else { //592
                                    $this->acmUpdateRateCCus(round($exValor, 2), $p_cdempresa, $p_cdlote, $result_adqRateCCus["NRLANCLOTE"],
                                        $cdsGeraLote_reg["CDFILIMOVIEST"], $cdsGeraLote_reg["CDCENTCUST"], $result_adqRateCCus["NRSEQLANLOTE"], $this->nrorg); //593
                                }
                            }
                        }
                    }
                    
                    //Conta contábil de Entrada do produto
                    $exValor *= -1; //609
                    
                    $result_adqLancLote = $this->factory->adqLancLote($p_cdlote, $cdsGeraLote_reg["CDCTBLENTRPR"], $p_cdempresa, $stNrLancLote, $this->nrorg); //610
                    if(count($result_adqLancLote) == 0) { //618
                        $stNrSequLote = str_pad($stNrSequLote + 1, 5, "0", STR_PAD_LEFT); //619
                        $stUltInsercao = $stNrSequLote;
                        $this->factory->acmLancLote($p_cdempresa, $p_cdlote, $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDCTBLENTRPR"], $p_cdhistpadr,
                            $dtContab, '9999999999', $this->getMessage($p_tipomovi), round($exValor, 2), $this->nrorg); //621
                        
                    } else { //635
                        $stNrSequLote = $result_adqLancLote["NRSEQLANLOTE"]; //636
                        $this->factory->acmUpdateLancLote(round($exValor, 2), $p_cdempresa, $p_cdlote, $stNrLancLote, $cdsGeraLote_reg["CDCTBLENTRPR"], $this->nrorg); //637
                        $stNrSequLote = $stUltInsercao; //645
                    }
                    
                    /*$cdsContaContabil = $this->factory->adqContaContabil($cdsGeraLote_reg["CDCONTCTBL"], $this->nrorg); //648
                    $cdsContaContabil = $cdsContaContabil[0]; //O adqContaContabil retornará apenas um resultado quando este tiver CDCONTCTBL setado*/
                    
                    if($cdsGeraLote_reg["CDCENTCUST"] != "") { //651
                        if($cdsContaContabil["IDCONTCCUS"] != 'O') { //652
                            if(count($result_adqLancLote) == 0) { //653
                                $this->factory->acmRatCusto($p_cdempresa, $p_cdlote, $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDFILIMOVIEST"],
                                    $cdsGeraLote_reg["CDCENTCUST"], $dtContab, round($exValor, 2), $this->nrorg); //654
                            } else { //666
                                $result_adqRateCCus = $this->factory->adqRateCCus($p_cdempresa, $p_cdlote, $stNrLancLote, $cdsGeraLote_reg["CDCTBLENTRPR"],
                                    $cdsGeraLote_reg["CDFILIMOVIEST"], $cdsGeraLote_reg["CDCENTCUST"], $this->nrorg); //667
                                if(count($result_adqRateCCus) == 0) { //677
                                    $this->factory->acmRatCusto($p_cdempresa, $p_cdlote, $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["CDFILIMOVIEST"],
                                    $cdsGeraLote_reg["CDCENTCUST"], $dtContab, round($exValor, 2), $this->nrorg); //678
                                } else {
                                    $this->acmUpdateRateCCus(round($exValor, 2), $p_cdempresa, $p_cdlote, $result_adqRateCCus["NRLANCLOTE"],
                                        $cdsGeraLote_reg["CDFILIMOVIEST"], $cdsGeraLote_reg["CDCENTCUST"], $result_adqRateCCus["NRSEQLANLOTE"], $this->nrorg); //691
                                }
                            }
                        }
                    }
                    $this->factory->acmUpdateItLancto2($p_cdempresa, $p_cdlote, $stNrLancLote, $stNrSequLote, $cdsGeraLote_reg["NRLANCMOVIEST"], $cdsGeraLote_reg["CDFILIMOVIEST"], $this->nrorg); //705
                } //714
            } //715
            //716
        } //Fim do laço principal
        
        $this->factory->commit(); //721
        
        return 'Contab_Ok';
    }
    
    
    private function formataData($data) {
        if (!$data) {
            return null;
        }

        // Verifica se está com o formato de data-hora.
        if (strlen($data) == 19) {
            $data = DateUtil::truncateString($data);
        }
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    public function geraRelatorioContabImpostos($row){
        try {
            $dtinicial  = DateUtil::getPrimeiroDiaDoMes( $this->formataData($row['DTREFINI']));
            $dtfinal    = DateUtil::getUltimoDiaDoMes( $this->formataData($row['DTREFINI']));
            $row['DTINICIAL'] = $dtinicial->format("d/m/Y");
            $row['DTFINAL'] = $dtfinal->format("d/m/Y");
            
            switch ($row['widgetName']) {
                case 'credito_presumido_pe':
                    $row['TIPO'] = '1';
                    $response = '1';
                    break;
                case 'credito_sp':
                    $row['TIPO'] = '2';
                    $response = '2';
                    break;
                case 'estorno_ba':
                    $row['TIPO'] = '3';
                    $response = '3';
                    break;
                case 'estorno_rj':
                    $row['TIPO'] = '4';
                    $response = '4';
                    break;
            }
            
            $existsPeriodoCtb = $this->factory->verificaPeriodoCtb($row);
            if(empty($existsPeriodoCtb)){
                throw new \Exception('Data de referência fora do período contábil');
            }
            
            $existsCdlote = $this->factory->verificaCdlote($row);
            if($existsCdlote){
                throw new \Exception('Código do lote já cadastrado');
            }
            
            $this->factory->deletaRelatorioInsc($row);
            
            $this->factory->contabilizaImp($row);
            
            return array("response" => $response);
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function geraRelatorioAjusteApuracaoFis($row){
        try {
            
            if(empty($row['CDINSCESTA'])){
                $row['CDINSCESTA'] = null;
            }
            
            if(empty($row['CDINSCMUNI'])){
                $row['CDINSCMUNI'] = null;
            }
            
            if(empty($row['CDHISTPADR'])){
                $row['CDHISTPADR'] = null;
            }
            
            $row['DTINICIAL'] = substr($row['DTINIPERIODO'], 0, 10);
            $row['DTFINAL'] = substr($row['DTFIMPERIODO'], 0, 10);
            
            $existsPeriodoCtb = $this->factory->verificaPeriodoCtb($row);
            if(empty($existsPeriodoCtb)){
                throw new \Exception('Data de referência fora do período contábil');
            }
            $existsCdlote = $this->factory->verificaCdlote($row);
            if($existsCdlote){
                throw new \Exception('Código do lote já cadastrado');
            }
            if(empty($row['CDCONTRATEIO'])){
                throw new \Exception('Padão de rateio não cadastrado');
            }
            //$this->factory->deletaRelatorioInsc($row);
            
            $this->factory->ctbAjusteApuracao($row);
            
            return array("sucesso" => true);
            
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function buscaAjustesMapa($row){
        try {
            $sql = "SELECT A.CDEMPRESA, A.CDCONTRATEIO, A.NRSEQUMAPAPU, A.IDCONTABILIZADO, A.NRORG, 'AC' AS IDTABELA, A.NRSEQAJUSTE, CASE A.IDINDICAJUSTE||A.IDTPAJUSTE WHEN 'C0' THEN 'RC' WHEN 'C1' THEN 'AC' WHEN 'DB0' THEN 'RDI' WHEN 'DB1' THEN 'ADI' WHEN 'DI0' THEN 'ARB' WHEN 'DI1' THEN 'ADB' END AS IDTPAJUSTE, A.DSAJUSTE, A.VRAJUSTE, A.CDCONTRATEIO, A.CDLOTE FROM AJUSTEAPURCONTR A WHERE A.NRSEQUMAPAPU = :P_NRSEQUMAPAPU AND A.IDFORMAPUR = 'C' AND A.IDINDICAJUSTE IS NOT NULL UNION ALL SELECT I.CDEMPRESA, I.CDCONTRATEIO, I.NRSEQUMAPAPU, I.IDCONTABILIZADO, I.NRORG, 'IM' AS IDTABELA, I.NRSEQITMAPR, I.IDTIPOITMAPR AS IDTPAJUSTE, I.DSITEMMAPR, I.VRITEMMAPR, I.CDCONTRATEIO, I.CDLOTE FROM ITMAPARECUP I WHERE I.NRSEQUMAPAPU = :P_NRSEQUMAPAPU AND I.IDTIPOITMAPR IN ('DD','DE','EC','ED','OC','OD') UNION ALL SELECT I.CDEMPRESA, I.CDCONTRATEIO, I.NRSEQUMAPAPU, I.IDCONTABILIZADO, I.NRORG, 'ID' AS IDTABELA, I.NRSEQITDETAPU, I.IDTIPODETAPU AS IDTPAJUSTE, I.DSITEMDETAPU, I.VRITEMDETAPU, I.CDCONTRATEIO, I.CDLOTE FROM ITDETAPUCONTR I WHERE I.NRSEQUMAPAPU = :P_NRSEQUMAPAPU AND I.IDTIPODETAPU = 'DD'";
            $connection = $this->entityManager->getConnection();
            $statement = $connection->prepare($sql);
            $statement->execute(array("P_NRSEQUMAPAPU" => $row['NRSEQUMAPAPU']));
            return $statement->fetchAll();
            
        } catch(\Exception $e) {
            throw $e;
        }
    }



    public function updateAjusteapurcontr($row){
         try {             
            $sql = "UPDATE AJUSTEAPURCONTR SET CDLOTE = :CDLOTE, CDEMPRESA = :CDEMPRESA WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU AND NRSEQAJUSTE = :NRSEQAJUSTE"; 
            $connection = $this->entityManager->getConnection();
            $statement = $connection->prepare($sql);
            $params = array(
                'CDLOTE'         => $row['CDLOTE'],
                'CDEMPRESA'      => $row['CDEMPRESA'],
                'NRSEQUMAPAPU'   => $row['NRSEQUMAPAPU'],
                'NRSEQAJUSTE'    => $row['NRSEQAJUSTE']
            );
            $statement->execute($params);
            //$connection->commit();
            //return $statement->fetchAll();

        } catch(xception $e) {
            throw $e;
        }        
    }
    
    public function updateItmaparecup($row){
         try {
            $sql = "UPDATE ITMAPARECUP SET CDLOTE = :CDLOTE, CDEMPRESA = :CDEMPRESA  WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU AND NRSEQITMAPR = :NRSEQITMAPR AND IDTIPOITMAPR = :IDTIPOITMAPR"; 
            $connection = $this->entityManager->getConnection();
            $statement = $connection->prepare($sql);
            $params = array(
                'CDLOTE'         => $row['CDLOTE'],
                'CDEMPRESA'      => $row['CDEMPRESA'],
                'NRSEQUMAPAPU'   => $row['NRSEQUMAPAPU'],
                'NRSEQITMAPR'    => $row['NRSEQAJUSTE'],
                'IDTIPOITMAPR'   => $row['IDTPAJUSTE']
            );
            $statement->execute($params);
            //$connection->commit();
            //return $statement->fetchAll();
                        //var_dump($params);
        } catch(xception $e) {
            throw $e;
        }        
    }
    
    public function updateItdetapucontr($row){
         try {
            $sql = "UPDATE ITDETAPUCONTR SET CDLOTE = :CDLOTE, CDEMPRESA = :CDEMPRESA WHERE NRSEQUMAPAPU = :NRSEQUMAPAPU AND NRSEQITDETAPU = :NRSEQITDETAPU AND IDTIPODETAPU = :IDTIPODETAPU"; 
            $connection = $this->entityManager->getConnection();
            $statement = $connection->prepare($sql);
            $params = array(
                'CDLOTE'         => $row['CDLOTE'],
                'CDEMPRESA'      => $row['CDEMPRESA'],
                'NRSEQUMAPAPU'   => $row['NRSEQUMAPAPU'],
                'NRSEQITDETAPU'  => $row['NRSEQAJUSTE'],
                'IDTIPODETAPU'   => $row['IDTPAJUSTE']
            );
            $statement->execute($params);
            //$connection->commit();
            //return $statement->fetchAll();
            
        } catch(xception $e) {
            throw $e;
        }        
    }
    
}